# Generated by Django 5.0.7 on 2024-07-18 10:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("sage_mailbox", "0006_alter_emailmessage_options_mailbox_folder_type"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mailbox",
            name="folder_type",
            field=models.CharField(
                choices=[
                    ("INBOX", "Inbox"),
                    ("SENT", "Sent"),
                    ("DRAFTS", "Drafts"),
                    ("SPAM", "Spam"),
                    ("TRASH", "Trash"),
                    ("CUSTOM", "Custom"),
                ],
                db_comment="The standardized type of the mailbox.",
                default="CUSTOM",
                help_text="The standardized type of the mailbox.",
                max_length=255,
                null=True,
                verbose_name="Folder Type",
            ),
        ),
    ]
