from .email import (
    download_as_eml,
    mark_as_flagged,
    mark_as_read,
    mark_as_unflagged,
    mark_as_unread,
    move_to_trash,
    restore_from_trash,
)
from .mailbox import delete_selected
