# Generated by Django 5.0.7 on 2024-07-17 19:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("sage_mailbox", "0003_alter_emailmessage_headers"),
    ]

    operations = [
        migrations.AlterField(
            model_name="emailmessage",
            name="message_id",
            field=models.CharField(
                blank=True,
                db_comment="Message ID of the email",
                editable=False,
                help_text="Message ID of the email",
                max_length=255,
                null=True,
                verbose_name="Message-ID Header",
            ),
        ),
    ]
