# Generated by Django 5.0.7 on 2024-07-17 18:16

from django.db import migrations, models

import sage_mailbox.validators


class Migration(migrations.Migration):

    dependencies = [
        ("sage_mailbox", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mailbox",
            name="name",
            field=models.CharField(
                db_comment="The unique name identifying the mailbox.",
                help_text="The unique name of the mailbox.",
                max_length=255,
                unique=True,
                validators=[sage_mailbox.validators.FolderNameValidator()],
                verbose_name="Name",
            ),
        ),
    ]
