# Generated by Django 5.0.7 on 2024-07-23 17:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("sage_mailbox", "0008_delete_inbox_alter_archive_options_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="emailmessage",
            options={
                "default_manager_name": "objects",
                "ordering": ["-date"],
                "permissions": [
                    ("mark_read", "Can mark email as read"),
                    ("mark_unread", "Can mark email as unread"),
                    ("flag_email", "Can flag email"),
                    ("unflag_email", "Can unflag email"),
                    ("download_eml", "Can download email data"),
                ],
                "verbose_name": "Inbox",
                "verbose_name_plural": "Inbox",
            },
        ),
    ]
