# Generated by Django 5.0.7 on 2024-07-17 19:44

import django_jsonform.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("sage_mailbox", "0004_alter_emailmessage_message_id"),
    ]

    operations = [
        migrations.AlterField(
            model_name="emailmessage",
            name="headers",
            field=django_jsonform.models.fields.JSONField(blank=True, null=True),
        ),
    ]
